//= require <dSS/devices/VisibilityMatrix>
//= require <dSS/devices/OutputModeForm>
//= require <dSS/devices/ScenePropertyForm>
//= require <dSS/devices/GroupPushbuttonForm>
//= require <dSS/devices/JokerConfigForm>
//= require <dSS/devices/HeatingConfigForm>
//= require <dSS/devices/ShadePropertyForm>
//= require <dSS/devices/HeatingPropertyForm>
//= require <dSS/devices/OperationTypeForm>
//= require <dSS/devices/MultiButtonConfigForm>
//= require <dSS/devices/RenameForm>
//= require <dSS/devices/ValveTypeForm>
//= require <dSS/devices/UMVRelayConfigForm>
//= require <dSS/devices/InputModeForm>
//= require <dSS/devices/OutputBlinkConfig>
//= require <dSS/devices/OutputImpulseConfig>
//
// parameters:
//      maskElement: element to mask when opening the window
//      iconCls: icon class for the title bar
//      extraTitle: string to append after the default title
//      standardForms []: array of forms for the standard view
//      advancedForms []: array of forms for the advanced view
//      name: device name
//      dSUID: device dSUID
//      showDefaults: show device name and dSUID
//      additionalInfo: key valu elements to render in addition to name and
//                      dSUID in the information panel
//      useTabs: bool, specifies if standard/advanced settings should be
//                displayed in a tab panel instead of a drop-down panel
//      tabHeight: mandatory in combination with useTabs, fixed height of the
//                 tab panel to prevent resize when jumping to next device
//      loaders []: array ofloader objects that will load
//                  data which has to be passed to all available forms
//
//                  forms that do not depend on the global information
//                  may load their internal data right away, others
//                  can wait for the "dataloaded" event that will be
//                  sent once all globalLoader objects have retreived
//                  their data


dSS.devprops.DevicePropertiesPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        var items = [];
        var standardItems = [];
        var advancedItems = [];
        var i;

        this.closed = false;
        this.disableSave = 0;
        this.cancelSubmit = false;
        if (config.showDefaults === true) {
            var info =
            [
                {
                    key: dSS._("Device name"),
                    value: dSS.util.getName(config.name)
                },
                {
                    key: dSS._("Device ID"),
                    value: config.DisplayID
                }
            ];

            if ((config.dSID) && (config.dSID.length > 0)) {
                info.push(
                {
                    key: dSS._("Device dSID"),
                    value: config.dSID
                });
            }

            if ((config.dSUID) && (config.dSUID.length > 0)) {
                info.push(
                {
                    key: dSS._("Device dSUID"),
                    value: config.dSUID
                });
            }

            if (config.additionalInfo != undefined) {
                info = info.concat(config.additionalInfo);
            }
            items.push(dSS.util.CreateKeyValPanel(info));
        }

        if (config.standardForms === undefined) {
            config.standardForms = [];
        }

        this.expectedChildEvents = config.standardForms.length;
        this.receivedSaveEvents = 0;
        this.receivedFormLoadEvents = 0;
        this.showSave = false;
        if ((config.standardForms.length > 1) || (config.useTabs)) {
            for (i = 0; i < config.standardForms.length; i++) {
                if (i === 0) {
                    config.standardForms[i].baseCls =
                                                    'x-panel-no-bottom-border';
                } else if (i === config.standardForms.length - 1) {
                    config.standardForms[i].baseCls = 'x-panel-no-top-border';
                } else {
                    config.standardForms[i].baseCls =
                            'x-panel-no-top-border x-panel-no-bottom-border';
                }
                if (config.useTabs) {
                    config.standardForms[i].baseCls +=
                        ' x-panel-no-left-border x-panel-no-right-border';
                }

                config.standardForms[i].baseCls += ' x-panel';
            }
        }
        if (config.standardForms.length > 0) {
            standardItems = standardItems.concat(config.standardForms);
        }

        this.loaders = [];
        this.loaderIndex = 0;
        this.expectedLoadEvents = 0;
        this.receivedLoadEvents = 0;

        if (config.loaders !== undefined) {
            this.loaders = config.loaders;
            this.expectedLoadEvents = config.loaders.length;
        }

        this.stayMasked = false;
        if (config.stayMasked === true) {
            this.stayMasked = true;
        }

        this.maskReason = dSS._("Configuration disabled");
        if (config.maskReason !== undefined) {
            this.maskReason = config.maskReason;
        }

        if (config.advancedForms !== undefined) {

            if ((config.advancedForms.length > 1) || (config.useTabs)) {
                for (i = 0; i < config.advancedForms.length; i++) {
                    if (i === 0) {
                        config.advancedForms[i].baseCls =
                                                    'x-panel-no-bottom-border';
                    } else if (i === config.advancedForms.length - 1) {
                        config.advancedForms[i].baseCls =
                                                    'x-panel-no-top-border';
                    } else {
                        config.advancedForms[i].baseCls =
                            'x-panel-no-top-border x-panel-no-bottom-border';
                    }

                    if (config.useTabs) {
                        config.advancedForms[i].baseCls +=
                            ' x-panel-no-left-border x-panel-no-right-border';
                    }

                    config.advancedForms[i].baseCls += ' x-panel';
                }
            }

            if (config.advancedForms.length > 0) {
                if (config.useTabs === true) {
                    advancedItems = advancedItems.concat(config.advancedForms);
                } else {
                    var advancedItems = [];
                    advancedItems.push({
                                    xtype: 'box',
                                    height: 10
                               });
                    advancedItems.push({
                                    hideMode:'offsets',
                                    xtype: 'panel',
                                    title: dSS._("Show advanced properties"),
                                    bubbleEvents: [ 'updateshadow',
                                                    'formretry',
                                                    'formchanged' ],
                                    collapsed: true,
                                    ref: 'advanced',
                                    collapsible: true,
                                    collapseFirst: true,
                                    animCollapse: false,
                                    floatable: false,
                                    titleCollapse: true,
                                    listeners:
                                    {
                                        collapse: function(p) {
                                            p.setTitle(
                                              dSS._("Show advanced properties"));
                                            this.fireEvent('updateshadow');
                                        },
                                        expand: function(p) {
                                            p.setTitle(
                                              dSS._("Hide advanced properties"));
                                            this.fireEvent('updateshadow');
                                        }
                                    },
                                    items: [ config.advancedForms ]
                            });
                }
                this.expectedChildEvents = this.expectedChildEvents +
                                           config.advancedForms.length;
            }
        }

        this.addEvents('formsubmitted', 'formloaded', 'loadcomplete',
                       'forcereload', 'sharedata', 'sharedevent', 'formretry',
                       'saveclicked', 'cancelclicked', 'setmask',
                       'formchanged', 'forcesave');

        if (config.useTabs === true) {
            this.standard = new Ext.Panel(
                {
                    xtype: 'panel',
                    title: dSS._("Settings"),
                    ref: 'standard',
                    bubbleEvents: [ 'formretry', 'formchanged', 'forcesave' ],
                    items: standardItems
                });

            var tabs = [ this.standard ];

            if (advancedItems.length > 0) {
                this.advanced = new Ext.Panel(
                    {
                        xtype: 'panel',
                        title: dSS._("Advanced settings"),
                        ref: 'advanced',
                        bubbleEvents: [
                                        'formretry', 'formchanged', 'forcesave'
                                      ],
                        items: advancedItems
                    });
                tabs.push(this.advanced);
            }

            items.push(new Ext.TabPanel({
                    xtype: 'tabpanel',
                    bubbleEvents: [ 'formretry', 'formchanged', 'forcesave' ],
                    ref: 'tabs',
                    border: true,
                    activeTab: 0,
                    height: config.tabHeight,
                    width: config.tabWidth,
                    id: 'devPropTabPanel',
                    items: tabs,
                    deferredRender: false,
                    forceLayout: true
                })
            );
        } else {
            items = items.concat(standardItems);
            items = items.concat(advancedItems);
        }

        Ext.apply(this, {
            layout: 'table',
            border: false,
            bubbleEvents:
            [
                'savebuttonstate', 'closewindow', 'updateshadow',
                'cancelbuttonstate', 'setmask', 'forcesave'
            ],
            layoutConfig: {
                columns: 1,
                tableAttrs: {
                    width: '100%'
                }
            },
            bodyCfg: {
                cls: 'x-window-body x-window-mc'
            },
            padding: 10,
            header: false,
            items: items
        });

        dSS.devprops.DevicePropertiesPanel.superclass.constructor.apply(this,
                arguments);

        this.on('formretry', function() {
                this.fireEvent('cancelbuttonstate', true);
                this.cancelSubmit = true;
                this.fireEvent('setmask', false);
            },
            this);

        this.on('formchanged', function() {
                this.fireEvent('savebuttonstate', true);
            },
            this);

        this.on('saveclicked', function() {
                this.fireEvent('setmask', true,
                               dSS._("Saving device scene settings") + '...');
                this.fireEvent('savebuttonstate', false);
                this.fireEvent('cancelbuttonstate', false);
                this.receivedSaveEvents = 0;
                this.cancelSubmit = false;
                if (this.advanced !== undefined) {
                    this.advanced.items.each(function(item) {
                        if (this.cancelSubmit) {
                            return;
                        }
                        item.fireEvent('submit');
                    });
                }
                if (this.standard !== undefined) {
                    this.standard.items.each(function(item) {
                        if (this.cancelSubmit) {
                            return;
                        }
                        item.fireEvent('submit');
                    });
                }

                this.items.each(function(item) {
                    if (this.cancelSubmit) {
                        return;
                    }
                    item.fireEvent('submit');
                });
            },
            this);

        this.on('cancelclicked', function() {
            this.abortLoading();
            this.closed = true;
            this.fireEvent('closewindow', 'cancel');
        },
        this);

        this.on('afterlayout',
            function() {
                if (this.loaders.length > 0) {
                    this.fireEvent('setmask', true,
                        dSS._("Loading properties") + '...');
                    this.loadNext();
                } else {
                    this.fireEvent('setmask', false);
                }
            },
            this);

        if (this.tabs !== undefined) {
            // different tabs may change the window height
            this.tabs.on('tabchange', function(tp, title) {
                this.fireEvent('updateshadow');
            },
            this);
        }

        this.on('formloaded',
            function(submitOK) {
                if (submitOK === true) {
                    this.showSave = true;
                }
                this.receivedFormLoadEvents++;
                if (this.receivedFormLoadEvents >= this.expectedChildEvents) {
                    this.receivedFormLoadEvents = 0;
                }
            },
            this);

        this.on('formsubmitted',
            function() {
                this.receivedSaveEvents++;
                if (this.receivedSaveEvents >= this.expectedChildEvents) {
                    this.closed = true;
                    this.fireEvent('closewindow', 'submit');
                }
            },
            this);

        this.on('forcereload', this.reloadData.createDelegate(this), this);

        this.on('close', function() {
                this.closed = true;
            },
            this);

        this.on('sharedata', function(name, data) {
                this.items.each(function(item) {
                    item.fireEvent('sharedevent', name, data);
                });
                if (this.advanced !== undefined) {
                    this.advanced.items.each(function(item) {
                        item.fireEvent('sharedevent', name, data);
                    });
                }

                if (this.standard !== undefined) {
                    this.standard.items.each(function(item) {
                        item.fireEvent('sharedevent', name, data);
                    });
                }
            },
            this);

        this.on('saveenable', function(id, enabled) {
                if (enabled) {
                    this.disableSave = this.disableSave & ~id;
                } else {
                    this.disableSave = this.disableSave | id;
                }
                this.fireEvent('savebuttonstate', (this.disableSave === 0));
            },
            this);
    },
    abortLoading: function() {
        if (this.loaders.length === 0) {
            return;
        }

        if ((this.loaderIndex < (this.loaders.length + 1)) &&
            (this.loaderIndex >= 0) &&
            (this.loaders[this.loaderIndex-1].tid !== undefined)) {
                Ext.Ajax.abort(this.loaders[this.loaderIndex-1].tid);
        }
    },
    reloadData: function() {
        if (this.closed === true) {
            return;
        }

        if (this.loading === true) {
            if (this.loaders.length > 0) {
                this.reloadData.defer(500, this);
            }
            return;
        }

        if (this.loaders.length > 0) {
            this.fireEvent('setmask', true,
                     dSS._("Loading properties") + '...');
            this.receivedFormLoadEvents = 0;
            this.receivedLoadEvents = 0;
            this.loaderIndex = 0;
            this.showSave = false;
            this.fireEvent('savebuttonstate', false);
            this.loadNext();
        }
    },
    loadNext: function() {
        if (this.closed === true) {
            return;
        }
        this.loading = true;
        if (this.loaderIndex < this.loaders.length) {
            this.loaders[this.loaderIndex].addListener('loadcomplete',
                                        this.loadComplete.createDelegate(this),
                                        this, { single: true });

            this.loaders[this.loaderIndex++].load();
        }
    },
    saveSettings: function(param1, param2) {
        this.receivedSaveEvents = 0;
        this.items.each(function(item) { item.fireEvent('submit'); });
        if (this.advanced !== undefined) {
            this.advanced.items.each(function(item) {
                item.fireEvent('dataloaded', param1, param2);
            });
        }

        if (this.standard !== undefined) {
            this.standard.items.each(function(item) {
                item.fireEvent('dataloaded', param1, param2);
            });
        }
    },
    loadComplete: function(data) {
        this.receivedLoadEvents++;
        this.items.each(function(item) {
            item.fireEvent('dataloaded', data);
        });

        if (this.advanced !== undefined) {
            this.advanced.items.each(function(item) {
                item.fireEvent('dataloaded', data);
            });
        }

        if (this.standard !== undefined) {
            this.standard.items.each(function(item) {
                item.fireEvent('dataloaded', data);
            });
        }

        if (this.receivedLoadEvents >= this.expectedLoadEvents) {
            this.loading = false;
            this.items.each(function(item) {
                item.fireEvent('loadcomplete');
            });

            if (this.advanced !== undefined) {
                this.advanced.items.each(function(item) {
                    item.fireEvent('loadcomplete');
                });
            }

            if (this.standard !== undefined) {
                this.standard.items.each(function(item) {
                    item.fireEvent('loadcomplete');
                });
            }
            if (this.stayMasked === true) {
                this.fireEvent('setmask', true);
                Ext.Msg.alert(dSS._("Attention"), this.maskReason);
            } else {
                this.fireEvent('setmask', false);
            }
        } else {
            this.loadNext();
        }
    }
});

Ext.reg('dssdevscenepropspanel', dSS.devprops.DevicePropertiesPanel);

dSS.devprops.DevicePropertiesWindow = Ext.extend(Ext.Window, {
    constructor: function(config) {
        var disableLeft = true;
        var disableRight = true;
        this.jumpMode = false;
        if (config.jumpMode !== undefined) {
            this.jumpMode = config.jumpMode;
        }

        this.rowSel = config.rowSel;
        var device;
        var store;
        if (this.rowSel.grid) {
            device = this.rowSel.getSelected();
            store = this.rowSel.grid.getStore();
            // prevent selection model caching
            device = store.getById(device.get('dsuid'));
        } else {
            device = this.rowSel.getSelectedRecords()[0];
            store = this.rowSel.getStore();
            // prevent selection model caching
            device = store.getById(device.get('id'));
        }

        this.devices = store.getRange();
        this.dialogType = config.dialogType;
        this.featureConfig = config.featureConfig;
        this.callerScope = config.callerScope;

        var props = dSS.devprops.CreatePropertiesPanel(config.dialogType,
                                    device, config.featureConfig,
                                    config.callerScope);
        this.properties = props.panel;

        var saveHidden = false;
        if (config.hideSaveButton !== undefined) {
            saveHidden = config.hideSaveButton
        }
        this.saveButton = new Ext.Button({
            text: dSS._("Save"),
            disabled: true,
            hidden: saveHidden,
            handler: this.sendSave.createDelegate(this)
        });

        this.cancelButton = new Ext.Button({
            xtype: 'button',
            id: 'cancelbutton',
            ref: 'cancelbutton',
            text: saveHidden || this.jumpMode ? dSS._("Close") : dSS._("Cancel"),
            scope: this,
            handler: this.sendCancel.createDelegate(this)
        });

        var buttons = [ this.saveButton, this.cancelButton ];
        var buttonAlign = 'right';

        if (this.jumpMode === true) {
            this.index = store.indexOf(device);

            if (this.devices.length > 1) {
                if (this.index > 0) {
                    disableLeft = false;
                }
                if (this.index < this.devices.length - 1) {
                    disableRight = false;
                }
            }

            buttonAlign = 'left';
            this.leftButton = new Ext.Button({
                iconCls: 'buttonLeftIcon',
                tooltip: dSS._("Select previous device"),
                width: 50,
                disabled: disableLeft,
                handler: this.previousDevice.createDelegate(this)
            });

            this.rightButton = new Ext.Button({
                iconCls: 'buttonRightIcon',
                tooltip: dSS._("Select next device"),
                width: 50,
                disabled: disableRight,
                handler: this.nextDevice.createDelegate(this)
            });

            buttons =
            [
                this.leftButton,
                this.rightButton,
                new Ext.Toolbar.Fill(),
                this.saveButton,
                this.cancelButton
            ];
        }

        this.defaultTitle = dSS._("Device properties");
        if (config.title !== undefined) {
            this.defaultTitle = config.title;
        }

        var title = this.defaultTitle;
        if (props.extraTitle !== undefined) {
            title = title + ' ' + props.extraTitle;
        }

        this.addEvents('saveclicked', 'cancelclicked', 'closewindow');

        var iconCls;
        if ((device) && device.get('dsuid')) {
            iconCls = 'titleIcon' + dSS.util.genCSSName(device);
        }
        Ext.apply(this, {
            layout: 'table',
            layoutConfig: {
                columns: 1,
                tableAttrs: {
                    width: '100%'
                }
            },
            bodyCfg: {
                cls: 'x-window-body x-window-mc'
            },
            border: false,
            resizable: false,
            iconCls: iconCls,
            title: title,
            items: [ this.properties ],
            buttonAlign: buttonAlign,
            buttons: buttons
        });

        if (config.maskElement) {
            config.maskElement.mask();
        }

        dSS.devprops.DevicePropertiesWindow.superclass.constructor.apply(this,
                arguments);

        this.on('afterrender', function() {
            this.properties.el.mask();
        },
        this);

        this.on('closewindow', function(reason) {
            if (this.jumpMode && (reason === 'submit')) {
                this.fireEvent('setmask', false );
                this.saveButton.disable();
                this.cancelButton.enable();
                this.handleJumpButtonsVisibility();
                return;
            }
            this.close();
        },
        this);

        this.on('close', function() {
            this.properties.getEl().unmask();
            if (this.maskElement) {
                this.maskElement.unmask();
            }
        },
        this);

        this.on('savebuttonstate', function(state) {
            if (this.saveButton.disabled !== (!state)) {
                this.saveButton.setDisabled(!state);
                if (this.jumpMode === true) {
                    if (this.afterSave !== true) {
                        if (state === false) {
                            this.handleJumpButtonsVisibility();
                        } else {
                            this.rightButton.disable();
                            this.leftButton.disable();
                        }
                    } else {
                        this.afterSave = false;
                    }
                }
            }
        },
        this);

        this.on('cancelbuttonstate', function(state) {
            this.cancelButton.setDisabled(!state);
        },
        this);

        this.on('updateshadow', function() {
            var s = this.getEl().shadow;
            if (s) {
                s.realign(this.x, this.y, this.getWidth(), this.getHeight());
            }
        },
        this);

        this.on('setmask', function(action, message) {
            if (action) {
                if (message) {
                    this.properties.el.mask(message, 'x-mask-loading');
                } else {
                    this.properties.el.mask();
                }
            } else {
                this.properties.el.unmask();
            }
        },
        this, { delay: 100 });


        this.forceSaveFunc = this.sendSave.createDelegate(this);
        this.properties.on('forcesave', this.forceSaveFunc);
    },
    sendSave: function() {
        this.afterSave = true;
        this.properties.fireEvent('saveclicked');
    },
    sendCancel: function() {
        this.properties.fireEvent('cancelclicked');
    },
    updateView: function() {
        var device;
        var store;
        this.properties.un('forcesave', this.forceSaveFunc);
        if (this.rowSel.grid) {
            this.rowSel.selectRow(this.index, false, false);
            device = this.rowSel.getSelected();
            store = this.rowSel.grid.getStore();
            // prevent selection model caching
            device = store.getById(device.get('dsuid'));
        } else {
            this.rowSel.select(this.index, false, false);
            device = this.rowSel.getSelectedRecords()[0];
            store = this.rowSel.getStore();
            // prevent selection model caching
            device = store.getById(device.get('id'));
        }

        var props = dSS.devprops.CreatePropertiesPanel(this.dialogType,
                                    device, this.featureConfig,
                                    this.callerScope);
        var title = this.defaultTitle;
        if (props.extraTitle !== undefined) {
            title = title + ' ' + props.extraTitle;
        }
        var iconCls;
        if ((device) && device.get('dsuid')) {
            iconCls = 'titleIcon' + dSS.util.genCSSName(device);
        }
        this.setTitle(title, iconCls);
        this.properties = props.panel;
        this.removeAll(true);
        this.add(this.properties);
        this.doLayout(false, true);
        this.fireEvent('updateshadow');
        this.saveButton.disable();
        this.properties.on('forcesave', this.forceSaveFunc);
    },
    previousDevice: function() {
        this.properties.el.mask();
        this.properties.abortLoading();
        if (this.index > 0) {
            this.index--;
            if (this.index == 0) {
                this.leftButton.disable();
            }

            if ((this.devices.length > 1) &&
                (this.index < (this.devices.length - 1))) {
                this.rightButton.enable();
            }
        }
        this.updateView();
    },
    nextDevice: function() {
        this.properties.el.mask();
        this.properties.abortLoading();
        if (this.index < (this.devices.length - 1)) {
            this.index++;
            if (this.index === (this.devices.length - 1)) {
                this.rightButton.disable();
            }

            if ((this.devices.length > 1) && (this.index > 0)) {
                this.leftButton.enable();
            }
        }
        this.updateView();
    },
    handleJumpButtonsVisibility: function() {
        if (this.index == 0) {
            this.leftButton.disable();
        }

        if ((this.devices.length > 1) &&
            (this.index < (this.devices.length - 1))) {
            this.rightButton.enable();
        }

        if (this.index === (this.devices.length - 1)) {
            this.rightButton.disable();
        }

        if ((this.devices.length > 1) && (this.index > 0)) {
            this.leftButton.enable();
        }
    }
});
