Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.OutputModeForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device = config.device;

        this.dSUID = this.device.get('dsuid');
        this.dialogType = config.type;
        this.group = config.group;
        this.matrix = config.matrix;
        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.loadedValue = undefined;

        var items = [];

        this.registeredTips = [];

        this.magicNumber_rb1 = 661;
        this.magicNumber_rb2 = 662;
        if (this.matrix.heatingoutmode === true) {
            items =
            [
                {
                    boxLabel: dSS._("switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: this.magicNumber_rb1.toString(),
                    inputValue: this.magicNumber_rb1
                },
                {
                    boxLabel: dSS._("pulse width modulation (PWM)"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: this.magicNumber_rb2.toString(),
                    inputValue: this.magicNumber_rb2
                },
                {
                    boxLabel: dSS._("disabled"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                    inputValue: dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                }
            ];
        } else if (this.matrix.extradimmer === true) {
            items =
            [
                {
                    boxLabel: dSS._("switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_SWITCH.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_SWITCH
                },
                {
                    boxLabel: dSS._("dimmed 0-10V"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DIMMER_0_10V.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DIMMER_0_10V
                },
                {
                    boxLabel: dSS._("dimmed 1-10V"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DIMMER_1_10V.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DIMMER_1_10V
                },
                {
                    boxLabel: dSS._("disabled"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                }
            ];
        } else if (this.matrix.umroutmode === true) {
            this.sswitchedText = dSS._("Both relais are switched independently.");
            items =
            [
                {
                    boxLabel: dSS._("single switched"),
                    ref: 'singleSwitchedRB',
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL
                }
            ];

            if (this.device.get('dSUIDIndex') < 3) {
                this.cswitchedText = dSS._("Both relais are switched toghether.");
                this.c2sswitchedText = dSS._("Both relais switch combined in two steps depending on the output value. Output > 33% - relais 1 is on. Output > 66% - relais 1 and relais 2 are on.");
                this.c3sswitchedText = dSS._("Both relais switch combined in three steps depending on the output value. Output > 25% - relais 1 is on. Output > 50% - relais 1 is off and relais 2 is on. Output > 75% - relais 1 and relais 2 are on.");
                items = items.concat(
                [
                    {
                        boxLabel: dSS._("combined switched"),
                        name: 'rb_outmode_' + config.dSUID,
                        id: dSS.config.deviceParams.OUTPUT_MODE_BIPOLAR_SWITCH.toString(),
                        inputValue : dSS.config.deviceParams.OUTPUT_MODE_BIPOLAR_SWITCH
                    },
                    {
                        boxLabel: dSS._("combined two stage switched"),
                        name: 'rb_outmode_' + config.dSUID,
                        id: dSS.config.deviceParams.OUTPUT_MODE_TWO_STAGE_SWITCH.toString(),
                        inputValue : dSS.config.deviceParams.OUTPUT_MODE_TWO_STAGE_SWITCH
                    },
                    {
                        boxLabel: dSS._("combined three stage switched"),
                        name: 'rb_outmode_' + config.dSUID,
                        id: dSS.config.deviceParams.OUTPUT_MODE_THREE_STAGE_SWITCH.toString(),
                        inputValue : dSS.config.deviceParams.OUTPUT_MODE_THREE_STAGE_SWITCH
                    }
                ]);
            }

            this.disabledText = dSS._("Both relais outputs are disabled.");
            items.push(
            {
                boxLabel: dSS._("disabled"),
                name: 'rb_outmode_' + config.dSUID,
                id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                inputValue : dSS.config.deviceParams.OUTPUT_MODE_DISABLED
            });
        } else if (this.matrix.outmodeswitch === true) {
            items =
            [
                {
                    boxLabel: dSS._("switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL
                },
                {
                    boxLabel: dSS._("disabled"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                }
            ];
        } else {
            items =
            [
                {
                    boxLabel: dSS._("switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_SWITCH.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_SWITCH
                },
                {
                    boxLabel: dSS._("dimmed"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DIMMER.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DIMMER
                },
                {
                    boxLabel: dSS._("disabled"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                }
            ];
        }

        this.radio = new Ext.form.RadioGroup(
        {
            bubbleEvents: [ 'change' ],
            fieldLabel: dSS._("Output mode"),
            columns: 1,
            items: items
        });


        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            scope: this,
            items: this.radio
        });

        dSS.devprops.OutputModeForm.superclass.constructor.apply(this, arguments);

        if (this.matrix.heatingoutmode === true) {
            dSS.MessageBus.addListener('heatinggroupformchanged',
                this.updateGroup, this);
        }

        this.on('loadcomplete', function() {
                this.on('change', function() {
                    this.fireEvent('formchanged');
                },
                this);
                if (this.sswitchedText) {
                    this.registerRBTip(this.sswitchedText,
                        dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL.toString());
                }

                if (this.cswitchedText) {
                    this.registerRBTip(this.cswitchedText,
                        dSS.config.deviceParams.OUTPUT_MODE_BIPOLAR_SWITCH.toString());
                }

                if (this.c2sswitchedText) {
                    this.registerRBTip(this.c2sswitchedText,
                        dSS.config.deviceParams.OUTPUT_MODE_TWO_STAGE_SWITCH.toString());
                }

                if (this.c3sswitchedText) {
                    this.registerRBTip(this.c3sswitchedText,
                        dSS.config.deviceParams.OUTPUT_MODE_THREE_STAGE_SWITCH.toString());
                }

                if (this.disabledText) {
                    this.registerRBTip(this.disabledText,
                        dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString());
                }
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.radio.markInvalid(data.json.message);
                    this.fireEvent('formloaded', true);
                    return;
                }

                var val = data.json.result[this.dSUID][0].outputMode;
                this.loadedValue = val;
                this.setRadioGroup(val);

                this.el.unmask();
                this.fireEvent('formloaded', true);
            },
            this);

        this.on('beforedestroy', function() {
                var i;
                for (i = 0; i < this.registeredTips.length; i++) {
                    Ext.QuickTips.unregister(this.registeredTips[i]);   
                }
                this.closed = true;
                if (this.matrix.heatingoutmode === true) {
                    dSS.MessageBus.removeListener('heatinggroupformchanged',
                        this.updateGroup, this);
                }
            },
            this);

        this.on('submit', function() {
            var value;
            // this happens if the value was not loaded correctly
            if (this.radio.getValue() != null) {
                value = this.radio.getValue().getRawValue();
                value = this.convertValueFromMagic(value);
            }

            // do not submit anything if value was not loaded and also has
            // not been changed by the user
            if ((value === undefined) || (value == this.loadedValue)) {
                this.fireEvent('formsubmitted');
                return;
            }

            var dSUID = this.dSUID;

            Ext.Ajax.request({
                url: '/json/device/setOutputMode',
                method: 'GET',
                disableCaching: true,
                scope: this,
                params:
                {
                    dsuid : dSUID,
                    modeID : value
                },
                scope: this,
                success: function(result, request) {
                    try {
                        var jsonData = Ext.util.JSON.decode(result.responseText);
                        if (jsonData.ok !== true) {
                            throw "error";
                        }

                        var reply = jsonData.result;

                        var store = dSS.data.Loader.getDeviceStore();
                        var record = store.getById(dSUID);
                        if (record !== undefined) {
                            record.set('outputMode', parseInt(value, 10));
                            record.commit();
                        }

                        var ms = dSS.data.Loader.getModulatorStore();

                        if (reply.action === "add") {
                            reply.device.firstSeen = Date.parseDate(
                                reply.device.firstSeen, "Y-m-d H:i:s",
                                true);
                            reply.device.inactiveSince = Date.parseDate(
                                reply.device.inactiveSince, "Y-m-d H:i:s",
                                true);

                            var zS = dSS.data.Loader.getZoneStore();
                            var index = zS.findExact('id', reply.device.zoneID);
                            if (index > -1) {
                                var z = zS.getAt(index);
                                reply.device.zoneName = z.get('name');
                                if ((reply.device.name === undefined) ||
                                    (reply.device.name.length === 0)) {
                                    z.set('unnamedDevices', true);
                                    z.commit();
                                }
                            }

                            var dsm = ms.getById(reply.device.meterDSUID);
                            if (dsm !== undefined) {
                                reply.device.meterName = dsm.get("name");
                                dsm.set('numDevices',
                                        dsm.get('numDevices') + 1);
                            }

                            var dev = { devices : [] };
                            dev.devices.push(reply.device);
                            store.loadData(dev, true);
                            dSS.MessageBus.fireEvent('scenesmarkdirty');
                        } else if (reply.action === "remove") {
                             index = store.find('dsuid', reply.device.dSUID);
                             if (index > -1) {
                                 store.removeAt(index);
                                 dSS.MessageBus.fireEvent('scenesmarkdirty');
                             }

                             var dsm = ms.getById(reply.device.meterDSUID);
                             if (dsm !== undefined) {
                                dsm.set('numDevices',
                                        dsm.get('numDevices') - 1);
                             }
                             dSS.data.Loader.getZoneStore().fireEvent(
                                     'zonedeviceleft', reply.device.zoneID);
                        }

                        dSS.MessageBus.fireEvent('scenesmarkdirty');
                        this.fireEvent('formsubmitted');
                    } catch (err) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not save output mode settings"));
                        this.fireEvent('formsubmitted');
                    }
                },
                failure: function(result, request) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not save output mode settings"));
                    this.fireEvent('formsubmitted');
                }
            });
        },
        this);
    },
    registerRBTip: function(text, rbid) {
        // the tooltips are ultimately a hack because radio button
        // tooltips are not supported by default
        var formel = Ext.get('x-form-el-' + rbid);
        if (formel) {
            var tipel = formel.first();
            if (tipel) {
                Ext.QuickTips.register({
                    text: text,
                    target: tipel.id,
                    trackMouse: false,
                    dismissDelay: 10000
                });
                this.registeredTips.push(tipel.id);
            }
        }
    },
    updateGroup: function(group) {
        if (this.matrix.heatingoutmode !== true) {
            return;
        }

        this.group = group;
    },
    setRadioGroup: function(value) {
        if (this.matrix.heatingoutmode === true) {
            if ((value === dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED) ||
                (value === dSS.config.deviceParams.OUTPUT_MODE_SWITCH)) {
                value = this.magicNumber_rb1;
            } else if (dSS.util.isHeatingPWMMode(value)) {
                value = this.magicNumber_rb2;
            }
        }

        this.radio.setValue(value, value);
    },

    convertValueFromMagic: function(value) {
        value = parseInt(value);

        if (this.matrix.heatingoutmode !== true) {
            return value;
        }

        if (value === this.magicNumber_rb1) {
            if (this.group === dSS.config.groups.TEMPCONTROL) {
                return dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED;
            } else {
                return dSS.config.deviceParams.OUTPUT_MODE_SWITCH;
            }
        } else if (value === this.magicNumber_rb2) {
            if (this.group === dSS.config.groups.TEMPCONTROL) {
                return dSS.config.deviceParams.OUTPUT_MODE_CONTROL_PWM;
            } else {
                return dSS.config.deviceParams.OUTPUT_MODE_PWM;
            }
        }

        return value; // 0 = disabled is same for both groups
    }
});

Ext.reg('dssdevoutputmodeform', dSS.devprops.OutputModeForm);

